@@ -20,9 +20,10 @@ import json
 from coredata import MesonException
 
 class TestSerialisation:
-    def __init__(self, name, fname, is_cross, exe_wrapper, is_parallel, cmd_args, env,
-                 should_fail, valgrind_args, timeout, extra_paths):
+    def __init__(self, name, suite, fname, is_cross, exe_wrapper, is_parallel, cmd_args, env,
+                 should_fail, valgrind_args, timeout, workdir, extra_paths):
         self.name = name
+        self.suite = suite
         self.fname = fname
         self.is_cross = is_cross
         self.exe_runner = exe_wrapper
@@ -32,6 +33,7 @@ class TestSerialisation:
         self.should_fail = should_fail
         self.valgrind_args = valgrind_args
         self.timeout = timeout
+        self.workdir = workdir
         self.extra_paths = extra_paths
 
 # This class contains the basic functionality that is needed by all backends.
@@ -44,6 +46,9 @@ class Backend():
         self.dep_rules = {}
         self.build_to_src = os.path.relpath(self.environment.get_source_dir(),
                                             self.environment.get_build_dir())
+        for t in self.build.targets:
+            priv_dirname = self.get_target_private_dir_abs(t)
+            os.makedirs(priv_dirname, exist_ok=True)
 
     def get_compiler_for_lang(self, lang):
         for i in self.build.compilers:
@@ -72,15 +77,14 @@ class Backend():
             dirname = target.get_subdir()
         else:
             dirname = 'meson-out'
-        os.makedirs(os.path.join(self.environment.get_build_dir(), dirname), exist_ok=True)
         return dirname
 
     def get_target_private_dir(self, target):
-        return os.path.join(self.get_target_dir(target), target.get_basename() + target.type_suffix())
+        dirname = os.path.join(self.get_target_dir(target), target.get_basename() + target.type_suffix())
+        return dirname
 
     def get_target_private_dir_abs(self, target):
-        dirname = self.get_target_private_dir(target)
-        os.makedirs(os.path.join(self.environment.get_build_dir(), dirname), exist_ok=True)
+        dirname = os.path.join(self.environment.get_build_dir(), self.get_target_private_dir(target))
         return dirname
 
     def generate_unity_files(self, target, unity_src):
@@ -127,24 +131,28 @@ class Backend():
         datafile = open(test_data, 'wb')
         self.write_test_file(datafile)
         datafile.close()
+        benchmark_data = os.path.join(self.environment.get_scratch_dir(), 'meson_benchmark_setup.dat')
+        datafile = open(benchmark_data, 'wb')
+        self.write_benchmark_file(datafile)
+        datafile.close()
+
+    def has_source_suffix(self, target, suffix):
+        for s in target.get_sources():
+            if s.endswith(suffix):
+                return True
+        return False
 
     def has_vala(self, target):
-        for s in target.get_sources():
-            if s.endswith('.vala'):
-                return True
-        return False
+        return self.has_source_suffix(target, '.vala')
 
     def has_rust(self, target):
-        for s in target.get_sources():
-            if s.endswith('.rs'):
-                return True
-        return False
+        return self.has_source_suffix(target, '.rs')
 
     def has_cs(self, target):
-        for s in target.get_sources():
-            if s.endswith('.cs'):
-                return True
-        return False
+        return self.has_source_suffix(target, '.cs')
+
+    def has_swift(self, target):
+        return self.has_source_suffix(target, '.swift')
 
     def determine_linker(self, target, src):
         if isinstance(target, build.StaticLibrary):
@@ -172,7 +180,7 @@ class Backend():
 
     def determine_ext_objs(self, extobj, proj_dir_to_build_root=''):
         result = []
-        targetdir = self.get_target_private_dir_abs(extobj.target)
+        targetdir = self.get_target_private_dir(extobj.target)
         suffix = '.' + self.environment.get_object_suffix()
         for osrc in extobj.srclist:
             osrc_base = osrc.fname
@@ -192,7 +200,7 @@ class Backend():
 
     def get_pch_include_args(self, compiler, target):
         args = []
-        pchpath = self.get_target_private_dir_abs(target)
+        pchpath = self.get_target_private_dir(target)
         includeargs = compiler.get_include_args(pchpath)
         for lang in ['c', 'cpp']:
             p = target.get_pch(lang)
@@ -269,9 +277,15 @@ class Backend():
                 result.append(dirseg)
         return result
 
+    def write_benchmark_file(self, datafile):
+        self.write_test_serialisation(self.build.get_benchmarks(), datafile)
+
     def write_test_file(self, datafile):
+        self.write_test_serialisation(self.build.get_tests(), datafile)
+
+    def write_test_serialisation(self, tests, datafile):
         arr = []
-        for t in self.build.get_tests():
+        for t in tests:
             exe = t.get_exe()
             if isinstance(exe, dependencies.ExternalProgram):
                 fname = exe.fullpath
@@ -291,38 +305,12 @@ class Backend():
                 if isinstance(a, mesonlib.File):
                     a = os.path.join(self.environment.get_build_dir(), a.rel_to_builddir(self.build_to_src))
                 cmd_args.append(a)
-            ts = TestSerialisation(t.get_name(), fname, is_cross, exe_wrapper,
+            ts = TestSerialisation(t.get_name(), t.suite, fname, is_cross, exe_wrapper,
                                    t.is_parallel, cmd_args, t.env, t.should_fail, t.valgrind_args,
-                                   t.timeout, extra_paths)
+                                   t.timeout, t.workdir, extra_paths)
             arr.append(ts)
         pickle.dump(arr, datafile)
 
-    def generate_pkgconfig_files(self):
-        for p in self.build.pkgconfig_gens:
-            outdir = self.environment.scratch_dir
-            fname = os.path.join(outdir, p.filebase + '.pc')
-            ofile = open(fname, 'w')
-            coredata = self.environment.get_coredata()
-            ofile.write('prefix=%s\n' % coredata.get_builtin_option('prefix'))
-            ofile.write('libdir=${prefix}/%s\n' % coredata.get_builtin_option('libdir'))
-            ofile.write('includedir=${prefix}/%s\n\n' % coredata.get_builtin_option('includedir'))
-            ofile.write('Name: %s\n' % p.name)
-            if len(p.description) > 0:
-                ofile.write('Description: %s\n' % p.description)
-            if len(p.version) > 0:
-                ofile.write('Version: %s\n' % p.version)
-            ofile.write('Libs: -L${libdir} ')
-            for l in p.libraries:
-                ofile.write('-l%s ' % l.name)
-            ofile.write('\n')
-            ofile.write('CFlags: ')
-            for h in p.subdirs:
-                if h == '.':
-                    h = ''
-                ofile.write(os.path.join('-I${includedir}', h))
-                ofile.write(' ')
-            ofile.write('\n')
-
 
     def generate_depmf_install(self, d):
         if self.build.dep_manifest_name is None:
@@ -370,6 +358,9 @@ class Backend():
         ofilenames = [os.path.join(self.get_target_dir(target), i) for i in target.output]
         srcs = []
         outdir = self.get_target_dir(target)
+        # Many external programs fail on empty arguments.
+        if outdir == '':
+            outdir = '.'
         if absolute_paths:
             outdir = os.path.join(self.environment.get_build_dir(), outdir)
         for i in target.sources:
