@@ -65,6 +65,7 @@ class Build:
         self.cross_compilers = []
         self.global_args = {}
         self.tests = []
+        self.benchmarks = []
         self.headers = []
         self.man = []
         self.data = []
@@ -72,7 +73,6 @@ class Build:
         self.static_cross_linker = None
         self.pot = []
         self.subprojects = {}
-        self.pkgconfig_gens = []
         self.install_scripts = []
         self.install_dirs = []
         self.dep_manifest_name = None
@@ -108,6 +108,9 @@ class Build:
     def get_tests(self):
         return self.tests
 
+    def get_benchmarks(self):
+        return self.benchmarks
+
     def get_headers(self):
         return self.headers
 
@@ -575,7 +578,7 @@ class Generator():
         return self.arglist
 
 class GeneratedList():
-    def __init__(self, generator):
+    def __init__(self, generator, extra_args=[]):
         if hasattr(generator, 'held_object'):
             generator = generator.held_object
         self.generator = generator
@@ -583,6 +586,7 @@ class GeneratedList():
         self.outfilelist = []
         self.outmap = {}
         self.extra_depends = []
+        self.extra_args = extra_args
 
     def add_file(self, newfile):
         self.infilelist.append(newfile)
@@ -602,6 +606,9 @@ class GeneratedList():
     def get_generator(self):
         return self.generator
 
+    def get_extra_args(self):
+        return self.extra_args
+
 class Executable(BuildTarget):
     def __init__(self, name, subdir, subproject, is_cross, sources, objects, environment, kwargs):
         super().__init__(name, subdir, subproject, is_cross, sources, objects, environment, kwargs)
@@ -922,20 +929,14 @@ class ConfigurationData():
     def keys(self):
         return self.values.keys()
 
-class PkgConfigGenerator():
-    def __init__(self, libraries, subdirs, name, description, version, filebase):
-        self.libraries = []
-        for l in libraries:
-            if hasattr(l, 'held_object'):
-                self.libraries.append(l.held_object)
-            else:
-                self.libraries.append(l)
-        self.headerdirs = {}
-        self.subdirs = subdirs
-        self.name = name
-        self.description = description
-        self.version = version
-        self.filebase = filebase
+# A bit poorly named, but this represents plain data files to copy
+# during install.
+class Data():
+    def __init__(self, in_sourcetree, source_subdir, sources, install_dir):
+        self.in_sourcetree = in_sourcetree
+        self.source_subdir = source_subdir
+        self.sources = sources
+        self.install_dir = install_dir
 
 class InstallScript:
     def __init__(self, cmd_arr):
