@@ -27,7 +27,8 @@ header_suffixes = ['h', 'hh', 'hpp', 'hxx', 'H', 'moc', 'vapi']
 cpp_suffixes = ['cc', 'cpp', 'cxx', 'h', 'hh', 'hpp', 'hxx', 'c++']
 c_suffixes = ['c']
 clike_suffixes = c_suffixes + cpp_suffixes
-obj_suffixes = ['o', 'obj']
+obj_suffixes = ['o', 'obj', 'res']
+lib_suffixes = ['a', 'lib', 'dll', 'dylib', 'so']
 
 def is_header(fname):
     if hasattr(fname, 'fname'):
@@ -47,6 +48,12 @@ def is_object(fname):
     suffix = fname.split('.')[-1]
     return suffix in obj_suffixes
 
+def is_library(fname):
+    if hasattr(fname, 'fname'):
+        fname = fname.fname
+    suffix = fname.split('.')[-1]
+    return suffix in lib_suffixes
+
 gnulike_buildtype_args = {'plain' : [],
                           'debug' : ['-g'],
                           'debugoptimized' : ['-O2', '-g'],
@@ -92,6 +99,11 @@ mono_buildtype_args = {'plain' : [],
                        'debugoptimized': ['-debug', '-optimize+'],
                        'release' : ['-optimize+']}
 
+swift_buildtype_args = {'plain' : [],
+                        'debug' : ['-g'],
+                        'debugoptimized': ['-g', '-O'],
+                        'release' : ['-O']}
+
 gnu_winlibs = ['-lkernel32', '-luser32', '-lgdi32', '-lwinspool', '-lshell32',
                '-lole32', '-loleaut32', '-luuid', '-lcomdlg32', '-ladvapi32']
 
@@ -172,6 +184,9 @@ class Compiler():
     def has_function(self, *args, **kwargs):
         raise EnvironmentException('Language %s does not support function checks.' % self.language)
 
+    def unixtype_flags_to_native(self, args):
+        return args
+
 class CCompiler(Compiler):
     def __init__(self, exelist, version, is_cross, exe_wrapper=None):
         super().__init__(exelist, version)
@@ -210,6 +225,9 @@ class CCompiler(Compiler):
     def get_dependency_gen_args(self, outtarget, outfile):
         return ['-MMD', '-MQ', outtarget, '-MF', outfile]
 
+    def depfile_for_object(self, objfile):
+        return objfile + '.' + self.get_depfile_suffix()
+
     def get_depfile_suffix(self):
         return 'd'
 
@@ -406,7 +424,12 @@ int someSymbolHereJustForFun;
             cmdlist = self.exe_wrapper + [exename]
         else:
             cmdlist = exename
-        pe = subprocess.Popen(cmdlist, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+        try:
+            pe = subprocess.Popen(cmdlist, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+        except Exception as e:
+            mlog.debug('Could not run: %s (error: %s)\n' % (cmdlist, e))
+            return RunResult(False)
+
         (so, se) = pe.communicate()
         so = so.decode()
         se = se.decode()
@@ -943,13 +966,105 @@ class RustCompiler(Compiler):
     def get_buildtype_args(self, buildtype):
         return rust_buildtype_args[buildtype]
 
+class SwiftCompiler(Compiler):
+    def __init__(self, exelist, version):
+        super().__init__(exelist, version)
+        self.version = version
+        self.id = 'llvm'
+        self.language = 'swift'
+        self.is_cross = False
+
+    def get_id(self):
+        return self.id
+
+    def get_linker_exelist(self):
+        return self.exelist
+
+    def name_string(self):
+        return ' '.join(self.exelist)
+
+    def needs_static_linker(self):
+        return True
+
+    def get_exelist(self):
+        return self.exelist
+
+    def get_werror_args(self):
+        return ['--fatal-warnings']
+
+    def get_language(self):
+        return self.language
+
+    def get_dependency_gen_args(self, outtarget, outfile):
+        return ['-emit-dependencies']
+
+    def depfile_for_object(self, objfile):
+        return os.path.splitext(objfile)[0] + '.' + self.get_depfile_suffix()
+
+    def get_depfile_suffix(self):
+        return 'd'
+
+    def get_output_args(self, target):
+        return ['-o', target]
+
+    def get_linker_output_args(self, target):
+        return ['-o', target]
+
+    def get_header_import_args(self, headername):
+        return ['-import-objc-header', headername]
+
+    def get_warn_args(self, level):
+        return []
+
+    def get_buildtype_args(self, buildtype):
+        return swift_buildtype_args[buildtype]
+
+    def get_buildtype_linker_args(self, buildtype):
+        return []
+
+    def get_std_exe_link_args(self):
+        return ['-emit-executable']
+
+    def get_module_args(self, modname):
+        return ['-module-name', modname]
+
+    def get_mod_gen_args(self):
+        return ['-emit-module']
+
+    def build_rpath_args(self, *args):
+        return [] # FIXME
+
+    def get_include_args(self, dirname):
+        return ['-I' + dirname]
+
+    def get_compile_only_args(self):
+        return ['-c']
+
+    def sanity_check(self, work_dir):
+        src = 'swifttest.swift'
+        source_name = os.path.join(work_dir, src)
+        output_name = os.path.join(work_dir, 'swifttest')
+        ofile = open(source_name, 'w')
+        ofile.write('''1 + 2
+''')
+        ofile.close()
+        pc = subprocess.Popen(self.exelist + ['-emit-executable', '-o', output_name, src], cwd=work_dir)
+        pc.wait()
+        if pc.returncode != 0:
+            raise EnvironmentException('Swift compiler %s can not compile programs.' % self.name_string())
+        if subprocess.call(output_name) != 0:
+            raise EnvironmentException('Executables created by Swift compiler %s are not runnable.' % self.name_string())
+
+    def can_compile(self, filename):
+        suffix = filename.split('.')[-1]
+        return suffix in ('swift')
+
 class VisualStudioCCompiler(CCompiler):
     std_warn_args = ['/W3']
     std_opt_args= ['/O2']
     vs2010_always_args = ['/nologo', '/showIncludes']
     vs2013_always_args = ['/nologo', '/showIncludes', '/FS']
 
-
     def __init__(self, exelist, version, is_cross, exe_wrap):
         CCompiler.__init__(self, exelist, version, is_cross, exe_wrap)
         self.id = 'msvc'
@@ -1051,6 +1166,14 @@ class VisualStudioCCompiler(CCompiler):
     def get_option_link_args(self, options):
         return options['c_winlibs'].value
 
+    def unixtype_flags_to_native(self, args):
+        result = []
+        for i in args:
+            if i.startswith('-L'):
+                i = '/LIBPATH:' + i[2:]
+            result.append(i)
+        return result
+
 class VisualStudioCPPCompiler(VisualStudioCCompiler):
     def __init__(self, exelist, version, is_cross, exe_wrap):
         VisualStudioCCompiler.__init__(self, exelist, version, is_cross, exe_wrap)
@@ -1306,8 +1429,8 @@ class GnuCPPCompiler(CPPCompiler):
         return get_gcc_soname_args(self.gcc_type, shlib_name, path, soversion)
 
     def get_options(self):
-        opts = {'cpp_std' : coredata.UserComboOption('cpp_std', 'C language standard to use',
-                                                     ['none', 'c++03', 'c++11', 'c++1y'],
+        opts = {'cpp_std' : coredata.UserComboOption('cpp_std', 'C++ language standard to use',
+                                                     ['none', 'c++03', 'c++11', 'c++14'],
                                                      'c++11')}
         if self.gcc_type == GCC_MINGW:
             opts.update({
@@ -1353,7 +1476,7 @@ class ClangCPPCompiler(CPPCompiler):
 
     def get_options(self):
         return {'cpp_std' : coredata.UserComboOption('cpp_std', 'C++ language standard to use',
-                                                   ['none', 'c++03', 'c++11', 'c++1y'],
+                                                   ['none', 'c++03', 'c++11', 'c++14'],
                                                    'c++11')}
 
     def get_option_compile_args(self, options):
@@ -1471,6 +1594,9 @@ end program prog
     def get_module_outdir_args(self, path):
         return ['-J' + path]
 
+    def depfile_for_object(self, objfile):
+        return objfile + '.' + self.get_depfile_suffix()
+
     def get_depfile_suffix(self):
         return 'd'
 
@@ -1659,6 +1785,9 @@ class VisualStudioLinker():
     def get_option_link_args(self, options):
         return []
 
+    def unixtype_flags_to_native(self, args):
+        return args
+
 class ArLinker():
     std_args = ['csr']
 
@@ -1695,3 +1824,6 @@ class ArLinker():
 
     def get_option_link_args(self, options):
         return []
+
+    def unixtype_flags_to_native(self, args):
+        return args
