@@ -14,7 +14,7 @@
 
 import pickle, os, uuid
 
-version = '0.27.0'
+version = '0.28.0'
 
 build_types = ['plain', 'debug', 'debugoptimized', 'release']
 layouts = ['mirror', 'flat']
@@ -215,6 +215,8 @@ forbidden_target_names = {'clean': None,
                           'all': None,
                           'test': None,
                           'test-valgrind': None,
+                          'test-': None,
+                          'benchmark': None,
                           'install': None,
                           'build.ninja': None,
                          }
