@@ -71,6 +71,7 @@ class PkgConfigDependency(Dependency):
 
     def __init__(self, name, environment, kwargs):
         Dependency.__init__(self)
+        self.is_libtool = False
         self.required = kwargs.get('required', True)
         if 'native' in kwargs and environment.is_cross_build():
             want_cross = not kwargs['native']
@@ -150,6 +151,7 @@ class PkgConfigDependency(Dependency):
                                            'but we could not compute the actual shared'
                                            'library path' % lib)
                     lib = shared_lib
+                    self.is_libtool = True
 
                 self.libs.append(lib)
 
@@ -537,10 +539,7 @@ class BoostDependency(Dependency):
     def get_win_link_args(self):
         args = []
         if self.boost_root:
-            # FIXME, these are in gcc format, not msvc.
-            # On the other hand, so are the args that
-            # pkg-config returns.
-            args.append('/LIBPATH:' + self.libdir)
+            args.append('-L' + self.libdir)
         for module in self.requested_modules:
             module = BoostDependency.name2lib.get(module, module)
             if module in self.lib_modules_mt:
@@ -1057,6 +1056,16 @@ class ExtraFrameworkDependency(Dependency):
     def found(self):
         return self.name is not None
 
+class ThreadDependency(Dependency):
+    def __init__(self, environment, kwargs):
+        super().__init__()
+        self.name = 'threads'
+        self.is_found = True
+        mlog.log('Dependency', mlog.bold(self.name), 'found:', mlog.green('YES'))
+
+    def need_threads(self):
+        return True
+
 def get_dep_identifier(name, kwargs):
     elements = [name]
     modlist = kwargs.get('modules', [])
@@ -1106,4 +1115,5 @@ packages = {'boost': BoostDependency,
             'wxwidgets' : WxDependency,
             'sdl2' : SDL2Dependency,
             'gl' : GLDependency,
+            'threads' : ThreadDependency,
            }
