@@ -19,6 +19,10 @@ import configparser
 
 build_filename = 'meson.build'
 
+class EnvironmentException(coredata.MesonException):
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+
 def find_coverage_tools():
     gcovr_exe = 'gcovr'
     lcov_exe = 'lcov'
@@ -79,8 +83,8 @@ class Environment():
 
         # List of potential compilers.
         if mesonlib.is_windows():
-            self.default_c = ['cl', 'cc', 'gcc']
-            self.default_cpp = ['cl', 'c++', 'g++']
+            self.default_c = ['cl', 'cc', 'gcc', 'clang']
+            self.default_cpp = ['cl', 'c++', 'g++', 'clang++']
         else:
             self.default_c = ['cc']
             self.default_cpp = ['c++']
@@ -141,6 +145,9 @@ class Environment():
     def is_object(self, fname):
         return is_object(fname)
 
+    def is_library(self, fname):
+        return is_library(fname)
+
     def merge_options(self, options):
         for (name, value) in options.items():
             if name not in self.coredata.user_options:
@@ -179,8 +186,8 @@ class Environment():
             except OSError:
                 continue
             (out, err) = p.communicate()
-            out = out.decode()
-            err = err.decode()
+            out = out.decode(errors='ignore')
+            err = err.decode(errors='ignore')
             vmatch = re.search(Environment.version_regex, out)
             if vmatch:
                 version = vmatch.group(0)
@@ -198,7 +205,7 @@ class Environment():
                 return GnuCCompiler(ccache + [compiler], version, gtype, is_cross, exe_wrap)
             if 'clang' in out:
                 return ClangCCompiler(ccache + [compiler], version, is_cross, exe_wrap)
-            if 'Microsoft' in out:
+            if 'Microsoft' in out or 'Microsoft' in err:
                 # Visual Studio prints version number to stderr but
                 # everything else to stdout. Why? Lord only knows.
                 version = re.search(Environment.version_regex, err).group()
@@ -228,8 +235,8 @@ class Environment():
                 except OSError:
                    continue
                 (out, err) = p.communicate()
-                out = out.decode()
-                err = err.decode()
+                out = out.decode(errors='ignore')
+                err = err.decode(errors='ignore')
 
                 version = 'unknown version'
                 vmatch = re.search(Environment.version_regex, out)
@@ -303,8 +310,8 @@ class Environment():
             except OSError:
                 continue
             (out, err) = p.communicate()
-            out = out.decode()
-            err = err.decode()
+            out = out.decode(errors='ignore')
+            err = err.decode(errors='ignore')
             vmatch = re.search(Environment.version_regex, out)
             if vmatch:
                 version = vmatch.group(0)
@@ -322,7 +329,7 @@ class Environment():
                 return GnuCPPCompiler(ccache + [compiler], version, gtype, is_cross, exe_wrap)
             if 'clang' in out:
                 return ClangCPPCompiler(ccache + [compiler], version, is_cross, exe_wrap)
-            if 'Microsoft' in out:
+            if 'Microsoft' in out or 'Microsoft' in err:
                 version = re.search(Environment.version_regex, err).group()
                 return VisualStudioCPPCompiler([compiler], version, is_cross, exe_wrap)
         raise EnvironmentException('Unknown compiler(s) "' + ', '.join(compilers) + '"')
@@ -341,8 +348,8 @@ class Environment():
         except OSError:
             raise EnvironmentException('Could not execute ObjC compiler "%s"' % ' '.join(exelist))
         (out, err) = p.communicate()
-        out = out.decode()
-        err = err.decode()
+        out = out.decode(errors='ignore')
+        err = err.decode(errors='ignore')
         vmatch = re.search(Environment.version_regex, out)
         if vmatch:
             version = vmatch.group(0)
@@ -371,8 +378,8 @@ class Environment():
         except OSError:
             raise EnvironmentException('Could not execute ObjC++ compiler "%s"' % ' '.join(exelist))
         (out, err) = p.communicate()
-        out = out.decode()
-        err = err.decode()
+        out = out.decode(errors='ignore')
+        err = err.decode(errors='ignore')
         vmatch = re.search(Environment.version_regex, out)
         if vmatch:
             version = vmatch.group(0)
@@ -394,8 +401,8 @@ class Environment():
         except OSError:
             raise EnvironmentException('Could not execute Java compiler "%s"' % ' '.join(exelist))
         (out, err) = p.communicate()
-        out = out.decode()
-        err = err.decode()
+        out = out.decode(errors='ignore')
+        err = err.decode(errors='ignore')
         vmatch = re.search(Environment.version_regex, err)
         if vmatch:
             version = vmatch.group(0)
@@ -412,8 +419,8 @@ class Environment():
         except OSError:
             raise EnvironmentException('Could not execute C# compiler "%s"' % ' '.join(exelist))
         (out, err) = p.communicate()
-        out = out.decode()
-        err = err.decode()
+        out = out.decode(errors='ignore')
+        err = err.decode(errors='ignore')
         vmatch = re.search(Environment.version_regex, out)
         if vmatch:
             version = vmatch.group(0)
@@ -430,7 +437,7 @@ class Environment():
         except OSError:
             raise EnvironmentException('Could not execute Vala compiler "%s"' % ' '.join(exelist))
         (out, _) = p.communicate()
-        out = out.decode()
+        out = out.decode(errors='ignore')
         vmatch = re.search(Environment.version_regex, out)
         if vmatch:
             version = vmatch.group(0)
@@ -447,7 +454,7 @@ class Environment():
         except OSError:
             raise EnvironmentException('Could not execute Rust compiler "%s"' % ' '.join(exelist))
         (out, _) = p.communicate()
-        out = out.decode()
+        out = out.decode(errors='ignore')
         vmatch = re.search(Environment.version_regex, out)
         if vmatch:
             version = vmatch.group(0)
@@ -457,6 +464,23 @@ class Environment():
             return RustCompiler(exelist, version)
         raise EnvironmentException('Unknown compiler "' + ' '.join(exelist) + '"')
 
+    def detect_swift_compiler(self):
+        exelist = ['swiftc']
+        try:
+            p = subprocess.Popen(exelist + ['-v'], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+        except OSError:
+            raise EnvironmentException('Could not execute Swift compiler "%s"' % ' '.join(exelist))
+        (_, err) = p.communicate()
+        err = err.decode(errors='ignore')
+        vmatch = re.search(Environment.version_regex, err)
+        if vmatch:
+            version = vmatch.group(0)
+        else:
+            version = 'unknown version'
+        if 'Swift' in err:
+            return SwiftCompiler(exelist, version)
+        raise EnvironmentException('Unknown compiler "' + ' '.join(exelist) + '"')
+
     def detect_static_linker(self, compiler):
         if compiler.is_cross:
             linker = self.cross_info.config['binaries']['ar']
@@ -478,8 +502,8 @@ class Environment():
         except OSError:
             raise EnvironmentException('Could not execute static linker "%s".' % linker)
         (out, err) = p.communicate()
-        out = out.decode()
-        err = err.decode()
+        out = out.decode(errors='ignore')
+        err = err.decode(errors='ignore')
         if '/OUT:' in out or '/OUT:' in err:
             return VisualStudioLinker([linker])
         if p.returncode == 0:
