@@ -290,18 +290,17 @@ class GeneratorHolder(InterpreterObject):
         self.methods.update({'process' : self.process_method})
 
     def process_method(self, args, kwargs):
-        if len(kwargs) > 0:
-            raise InvalidArguments('Process does not take keyword arguments.')
         check_stringlist(args)
-        gl = GeneratedListHolder(self)
+        extras = mesonlib.stringlistify(kwargs.get('extra_args', []))
+        gl = GeneratedListHolder(self, extras)
         [gl.add_file(os.path.join(self.interpreter.subdir, a)) for a in args]
         return gl
 
 class GeneratedListHolder(InterpreterObject):
-    def __init__(self, arg1):
+    def __init__(self, arg1, extra_args=[]):
         super().__init__()
         if isinstance(arg1, GeneratorHolder):
-            self.held_object = build.GeneratedList(arg1.held_object)
+            self.held_object = build.GeneratedList(arg1.held_object, extra_args)
         else:
             self.held_object = arg1
 
@@ -400,29 +399,25 @@ class Headers(InterpreterObject):
     def get_custom_install_dir(self):
         return self.custom_install_dir
 
-class Data(InterpreterObject):
+class DataHolder(InterpreterObject):
     def __init__(self, in_sourcetree, source_subdir, sources, kwargs):
-        InterpreterObject.__init__(self)
-        self.in_sourcetree = in_sourcetree
-        self.source_subdir = source_subdir
-        self.sources = sources
-        kwsource = kwargs.get('sources', [])
-        if not isinstance(kwsource, list):
-            kwsource = [kwsource]
-        self.sources += kwsource
-        check_stringlist(self.sources)
-        self.install_dir = kwargs.get('install_dir', None)
-        if not isinstance(self.install_dir, str):
+        super().__init__()
+        kwsource = mesonlib.stringlistify(kwargs.get('sources', []))
+        sources += kwsource
+        check_stringlist(sources)
+        install_dir = kwargs.get('install_dir', None)
+        if not isinstance(install_dir, str):
             raise InterpreterException('Custom_install_dir must be a string.')
+        self.held_object = build.Data(in_sourcetree, source_subdir, sources, install_dir)
 
     def get_source_subdir(self):
-        return self.source_subdir
+        return self.held_object.source_subdir
 
     def get_sources(self):
-        return self.sources
+        return self.held_object.sources
 
     def get_install_dir(self):
-        return self.install_dir
+        return self.held_object.install_dir
 
 class InstallDir(InterpreterObject):
     def __init__(self, source_subdir, installable_subdir, install_dir):
@@ -529,9 +524,10 @@ class RunTargetHolder(InterpreterObject):
         self.held_object = build.RunTarget(name, command, args, subdir)
 
 class Test(InterpreterObject):
-    def __init__(self, name, exe, is_parallel, cmd_args, env, should_fail, valgrind_args, timeout):
+    def __init__(self, name, suite, exe, is_parallel, cmd_args, env, should_fail, valgrind_args, timeout, workdir):
         InterpreterObject.__init__(self)
         self.name = name
+        self.suite = suite
         self.exe = exe
         self.is_parallel = is_parallel
         self.cmd_args = cmd_args
@@ -539,6 +535,7 @@ class Test(InterpreterObject):
         self.should_fail = should_fail
         self.valgrind_args = valgrind_args
         self.timeout = timeout
+        self.workdir = workdir
 
     def get_exe(self):
         return self.exe
@@ -774,7 +771,6 @@ class ModuleHolder(InterpreterObject):
         state.targets = self.interpreter.build.targets
         state.headers = self.interpreter.build.get_headers()
         state.man = self.interpreter.build.get_man()
-        state.pkgconfig_gens = self.interpreter.build.pkgconfig_gens
         state.global_args = self.interpreter.build.global_args
         value = fn(state, args, kwargs)
         return self.interpreter.module_method_callback(value)
@@ -872,7 +868,7 @@ class MesonMain(InterpreterObject):
         self.build.dep_manifest_name = args[0]
 
     def project_version_method(self, args, kwargs):
-        return self.build.dep_manifest[self.interpreter.active_projectname]
+        return self.build.dep_manifest[self.interpreter.active_projectname]['version']
 
 class Interpreter():
 
@@ -944,6 +940,7 @@ class Interpreter():
                       'run_target' : self.func_run_target,
                       'generator' : self.func_generator,
                       'test' : self.func_test,
+                      'benchmark' : self.func_benchmark,
                       'install_headers' : self.func_install_headers,
                       'install_man' : self.func_install_man,
                       'subdir' : self.func_subdir,
@@ -961,7 +958,6 @@ class Interpreter():
                       'option' : self.func_option,
                       'get_option' : self.func_get_option,
                       'subproject' : self.func_subproject,
-                      'pkgconfig_gen' : self.func_pkgconfig_gen,
                       'vcs_tag' : self.func_vcs_tag,
                       'set_variable' : self.func_set_variable,
                       'is_variable' : self.func_is_variable,
@@ -1003,6 +999,8 @@ class Interpreter():
                 self.build.targets[v.name] = v
             elif isinstance(v, build.InstallScript):
                 self.build.install_scripts.append(v)
+            elif isinstance(v, build.Data):
+                self.build.data.append(v)
             else:
                 print(v)
                 raise InterpreterException('Module returned a value of unknown type.')
@@ -1150,7 +1148,7 @@ class Interpreter():
             raise InvalidCode('First argument to set_variable must be a string.')
         if not self.is_assignable(variable):
             raise InvalidCode('Assigned value not of assignable type.')
-        if re.fullmatch('[_a-zA-Z][_0-9a-zA-Z]*', varname) is None:
+        if re.match('[_a-zA-Z][_0-9a-zA-Z]*$', varname) is None:
             raise InvalidCode('Invalid variable name: ' + varname)
         if varname in self.builtin:
             raise InvalidCode('Tried to overwrite internal variable "%s"' % varname)
@@ -1244,36 +1242,6 @@ class Interpreter():
     def func_option(self, nodes, args, kwargs):
         raise InterpreterException('Tried to call option() in build description file. All options must be in the option file.')
 
-    def func_pkgconfig_gen(self, nodes, args, kwargs):
-        if len(args) > 0:
-            raise InterpreterException('Pkgconfig_gen takes no positional arguments.')
-        libs = kwargs.get('libraries', [])
-        if not isinstance(libs, list):
-            libs = [libs]
-        for l in libs:
-            if not (isinstance(l, SharedLibraryHolder) or isinstance(l, StaticLibraryHolder)):
-                raise InterpreterException('Library argument not a library object.')
-        subdirs = kwargs.get('subdirs', ['.'])
-        if not isinstance(subdirs, list):
-            subdirs = [subdirs]
-        for h in subdirs:
-            if not isinstance(h, str):
-                raise InterpreterException('Header argument not string.')
-        version = kwargs.get('version', '')
-        if not isinstance(version, str):
-            raise InterpreterException('Version must be a string.')
-        name = kwargs.get('name', None)
-        if not isinstance(name, str):
-            raise InterpreterException('Name not specified.')
-        filebase = kwargs.get('filebase', name)
-        if not isinstance(filebase, str):
-            raise InterpreterException('Filebase must be a string.')
-        description = kwargs.get('description', None)
-        if not isinstance(description, str):
-            raise InterpreterException('Description is not a string.')
-        p = build.PkgConfigGenerator(libs, subdirs, name, description, version, filebase)
-        self.build.pkgconfig_gens.append(p)
-
     @stringArgs
     def func_subproject(self, nodes, args, kwargs):
         if len(args) != 1:
@@ -1365,7 +1333,6 @@ class Interpreter():
                 newoptions = [option] + self.environment.cmd_line_options.projectoptions
                 self.environment.cmd_line_options.projectoptions = newoptions
 
-
     @stringArgs
     def func_project(self, node, args, kwargs):
         if len(args) < 2:
@@ -1377,7 +1344,9 @@ class Interpreter():
                 self.parse_default_options(kwargs['default_options'])
         self.active_projectname = args[0]
         self.project_version = kwargs.get('version', 'undefined')
-        self.build.dep_manifest[args[0]] = self.project_version
+        proj_license = mesonlib.stringlistify(kwargs.get('license', 'unknown'))
+        self.build.dep_manifest[args[0]] = {'version': self.project_version,
+                                            'license': proj_license}
         if self.subproject in self.build.projects:
             raise InvalidCode('Second call to project().')
         if not self.is_subproject() and 'subproject_dir' in kwargs:
@@ -1394,7 +1363,7 @@ class Interpreter():
         langs = self.coredata.compilers.keys()
         if 'vala' in langs:
             if not 'c' in langs:
-                raise InterpreterException('Compiling Vala requires a C compiler')
+                raise InterpreterException('Compiling Vala requires C. Add C to your project languages and rerun Meson.')
 
     @noKwargs
     @stringArgs
@@ -1472,6 +1441,11 @@ class Interpreter():
                     comp = self.environment.detect_fortran_compiler(False)
                     if need_cross_compiler:
                         cross_comp = self.environment.detect_fortran_compiler(True)
+                elif lang == 'swift':
+                    comp = self.environment.detect_swift_compiler()
+                    if need_cross_compiler:
+                        raise InterpreterException('Cross compilation with Swift is not working yet.')
+                        #cross_comp = self.environment.detect_fortran_compiler(True)
                 else:
                     raise InvalidCode('Tried to use unknown language "%s".' % lang)
                 comp.sanity_check(self.environment.get_scratch_dir())
@@ -1488,7 +1462,7 @@ class Interpreter():
                     for cmd_arg in self.environment.cmd_line_options.projectoptions:
                         if cmd_arg.startswith(cmd_prefix):
                             value = cmd_arg.split('=', 1)[1]
-                            new_options[i].value = value
+                            new_options[i].set_value(value)
                 new_options.update(self.coredata.compiler_options)
                 self.coredata.compiler_options = new_options
             mlog.log('Native %s compiler: ' % lang, mlog.bold(' '.join(comp.get_exelist())), ' (%s %s)' % (comp.id, comp.version), sep='')
@@ -1676,7 +1650,13 @@ class Interpreter():
         self.generators.append(gen)
         return gen
 
+    def func_benchmark(self, node, args, kwargs):
+        self.add_test(node, args, kwargs, False)
+
     def func_test(self, node, args, kwargs):
+        self.add_test(node, args, kwargs, True)
+
+    def add_test(self, node, args, kwargs, is_base_test):
         if len(args) != 2:
             raise InterpreterException('Incorrect number of arguments')
         if not isinstance(args[0], str):
@@ -1715,11 +1695,29 @@ class Interpreter():
         if not isinstance(should_fail, bool):
             raise InterpreterException('Keyword argument should_fail must be a boolean.')
         timeout = kwargs.get('timeout', 30)
+        if 'workdir' in kwargs:
+            workdir = kwargs['workdir']
+            if not isinstance(workdir, str):
+                raise InterpreterException('Workdir keyword argument must be a string.')
+            if not os.path.isabs(workdir):
+                raise InterpreterException('Workdir keyword argument must be an absolute path.')
+        else:
+            workdir = None
         if not isinstance(timeout, int):
             raise InterpreterException('Timeout must be an integer.')
-        t = Test(args[0], args[1].held_object, par, cmd_args, env, should_fail, valgrind_args, timeout)
-        self.build.tests.append(t)
-        mlog.debug('Adding test "', mlog.bold(args[0]), '".', sep='')
+        suite = mesonlib.stringlistify(kwargs.get('suite', ''))
+        if self.is_subproject():
+            newsuite = []
+            for s in suite:
+                newsuite.append(self.subproject.replace(' ', '_').replace('.', '_') + '.' + s)
+            suite = newsuite
+        t = Test(args[0], suite, args[1].held_object, par, cmd_args, env, should_fail, valgrind_args, timeout, workdir)
+        if is_base_test:
+            self.build.tests.append(t)
+            mlog.debug('Adding test "', mlog.bold(args[0]), '".', sep='')
+        else:
+            self.build.benchmarks.append(t)
+            mlog.debug('Adding benchmark "', mlog.bold(args[0]), '".', sep='')
 
     @stringArgs
     def func_install_headers(self, node, args, kwargs):
@@ -1768,8 +1766,8 @@ class Interpreter():
 
     @stringArgs
     def func_install_data(self, node, args, kwargs):
-        data = Data(True, self.subdir, args, kwargs)
-        self.build.data.append(data)
+        data = DataHolder(True, self.subdir, args, kwargs)
+        self.build.data.append(data.held_object)
         return data
 
     @stringArgs
@@ -1819,7 +1817,7 @@ class Interpreter():
         else:
             raise InterpreterException('Configure_file must have either "configuration" or "command".')
         if isinstance(kwargs.get('install_dir', None), str):
-            self.build.data.append(Data(False, self.subdir, [output], kwargs))
+            self.build.data.append(DataHolder(False, self.subdir, [output], kwargs).held_object)
         return mesonlib.File.from_built_file(self.subdir, output)
 
     @stringArgs
