@@ -26,6 +26,8 @@ parser.add_argument('--wrapper', default=None, dest='wrapper',
                     help='wrapper to run tests with (e.g. valgrind)')
 parser.add_argument('--wd', default=None, dest='wd',
                     help='directory to cd into before running')
+parser.add_argument('--suite', default=None, dest='suite',
+                    help='Only run tests belonging to this suite.')
 parser.add_argument('args', nargs='+')
 
 
@@ -104,7 +106,7 @@ def run_single_test(wrap, test):
         if len(test.extra_paths) > 0:
             child_env['PATH'] = child_env['PATH'] + ';'.join([''] + test.extra_paths)
         p = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE,
-                             env=child_env)
+                             env=child_env, cwd=test.workdir)
         timed_out = False
         try:
             (stdo, stde) = p.communicate(timeout=test.timeout)
@@ -144,6 +146,11 @@ def drain_futures(futures):
         (result, numlen, tests, name, i, logfile, jsonlogfile) = i
         print_stats(numlen, tests, name, result.result(), i, logfile, jsonlogfile)
 
+def filter_tests(suite, tests):
+    if suite is None:
+        return tests
+    return [x for x in tests if suite in x.suite]
+
 def run_tests(options, datafilename):
     logfile_base = 'meson-logs/testlog'
     if options.wrapper is None:
@@ -173,15 +180,24 @@ def run_tests(options, datafilename):
         num_workers = multiprocessing.cpu_count()
     executor = conc.ThreadPoolExecutor(max_workers=num_workers)
     futures = []
-    for i, test in enumerate(tests):
+    filtered_tests = filter_tests(options.suite, tests)
+    for i, test in enumerate(filtered_tests):
+        if test.suite[0] == '':
+            visible_name = test.name
+        else:
+            if options.suite is not None:
+                visible_name = options.suite + ' / ' + test.name
+            else:
+                visible_name = test.suite[0] + ' / ' + test.name
+
         if not test.is_parallel:
             drain_futures(futures)
             futures = []
             res = run_single_test(wrap, test)
-            print_stats(numlen, tests, test.name, res, i, logfile, jsonlogfile)
+            print_stats(numlen, filtered_tests, visible_name, res, i, logfile, jsonlogfile)
         else:
             f = executor.submit(run_single_test, wrap, test)
-            futures.append((f, numlen, tests, test.name, i, logfile, jsonlogfile))
+            futures.append((f, numlen, filtered_tests, visible_name, i, logfile, jsonlogfile))
     drain_futures(futures)
     print('\nFull log written to %s.' % logfilename)
 
