@@ -180,7 +180,10 @@ class Conf:
             self.print_aligned(optarr)
 
 if __name__ == '__main__':
-    options = parser.parse_args()
+    args = mesonlib.expand_arguments(sys.argv[:])
+    if not args:
+        sys.exit(1)
+    options = parser.parse_args(args[1:])
     if len(options.directory) > 1:
         print('%s <build directory>' % sys.argv[0])
         print('If you omit the build directory, the current directory is substituted.')
