@@ -22,6 +22,7 @@ from meson_install import InstallData
 from build import InvalidArguments
 from coredata import MesonException
 import os, sys, pickle, re
+import subprocess, shutil
 
 if mesonlib.is_windows():
     quote_char = '"'
@@ -129,16 +130,43 @@ class NinjaBackend(backends.Backend):
                 raise MesonException('Multiple producers for Ninja target "%s". Please rename your targets.' % n)
             self.all_outputs[n] = True
 
+    def detect_vs_dep_prefix(self, outfile, tempfilename):
+        '''VS writes its dependency in a locale dependent format.
+        Detect the search prefix to use.'''
+        if shutil.which('cl') is None:
+            return outfile
+        outfile.close()
+        open(os.path.join(self.environment.get_scratch_dir(), 'incdetect.c'),
+              'w').write('''#include<stdio.h>
+int dummy;
+''')
+
+        pc = subprocess.Popen(['cl', '/showIncludes', '/c', 'incdetect.c'],
+                              stdout=subprocess.PIPE,
+                              stderr=subprocess.PIPE,
+                              cwd=self.environment.get_scratch_dir())
+
+        (stdo, _) = pc.communicate()
+
+        for line in stdo.split(b'\r\n'):
+            if line.endswith(b'stdio.h'):
+                matchstr = b':'.join(line.split(b':')[0:2]) + b':'
+                binfile = open(tempfilename, 'ab')
+                binfile.write(b'msvc_deps_prefix = ' + matchstr + b'\r\n')
+                binfile.close()
+                return open(tempfilename, 'a')
+        raise MesonException('Could not determine vs dep dependency prefix string.')
+
     def generate(self, interp):
         self.interpreter = interp
         outfilename = os.path.join(self.environment.get_build_dir(), self.ninja_filename)
         tempfilename = outfilename + '~'
         outfile = open(tempfilename, 'w')
-        self.generate_pkgconfig_files()
         outfile.write('# This is the build file for project "%s"\n' % self.build.get_project())
         outfile.write('# It is autogenerated by the Meson build system.\n')
         outfile.write('# Do not edit by hand.\n\n')
         outfile.write('ninja_required_version = 1.5.1\n\n')
+        outfile = self.detect_vs_dep_prefix(outfile, tempfilename)
         self.generate_rules(outfile)
         self.generate_phony(outfile)
         outfile.write('# Build rules for targets\n\n')
@@ -159,6 +187,14 @@ class NinjaBackend(backends.Backend):
         # fully created.
         outfile.close()
         os.replace(tempfilename, outfilename)
+        self.generate_compdb()
+
+    # http://clang.llvm.org/docs/JSONCompilationDatabase.html
+    def generate_compdb(self):
+        ninja_exe = environment.detect_ninja()
+        builddir = self.environment.get_build_dir()
+        jsondb = subprocess.check_output([ninja_exe, '-t', 'compdb', 'c_COMPILER', 'cpp_COMPILER'], cwd=builddir)
+        open(os.path.join(builddir, 'compile_commands.json'), 'wb').write(jsondb)
 
     # Get all generated headers. Any source file might need them so
     # we need to add an order dependency to them.
@@ -195,6 +231,9 @@ class NinjaBackend(backends.Backend):
             return
         if 'vala' in self.environment.coredata.compilers.keys() and self.has_vala(target):
             gen_src_deps += self.generate_vala_compile(target, outfile)
+        if 'swift' in self.environment.coredata.compilers.keys() and self.has_swift(target):
+            self.generate_swift_target(target, outfile)
+            return
         self.scan_fortran_module_outputs(target)
         # The following deals with C/C++ compilation.
         (gen_src, gen_other_deps) = self.process_dep_gens(outfile, target)
@@ -225,6 +264,8 @@ class NinjaBackend(backends.Backend):
                                                                          header_deps))
                     elif self.environment.is_object(src):
                         obj_list.append(src)
+                    elif self.environment.is_library(src):
+                        pass
                     else:
                         # Assume anything not specifically a source file is a header. This is because
                         # people generate files with weird suffixes (.inc, .fh) that they then include
@@ -233,13 +274,13 @@ class NinjaBackend(backends.Backend):
             else:
                 for src in gensource.get_outfilelist():
                     if self.environment.is_object(src):
-                        obj_list.append(os.path.join(self.get_target_private_dir_abs(target), src))
+                        obj_list.append(os.path.join(self.get_target_private_dir(target), src))
                     elif not self.environment.is_header(src):
                         if is_unity:
                             if self.has_dir_part(src):
                                 rel_src = src
                             else:
-                                rel_src = os.path.join(self.get_target_private_dir_abs(target), src)
+                                rel_src = os.path.join(self.get_target_private_dir(target), src)
                             unity_deps.append(rel_src)
                             abs_src = os.path.join(self.environment.get_build_dir(), rel_src)
                             unity_src.append(abs_src)
@@ -430,7 +471,6 @@ class NinjaBackend(backends.Backend):
         self.generate_man_install(d)
         self.generate_data_install(d)
         self.generate_po_install(d, elem)
-        self.generate_pkgconfig_install(d)
         self.generate_custom_install_script(d)
         self.generate_subdir_install(d)
         elem.write(outfile)
@@ -467,17 +507,6 @@ class NinjaBackend(backends.Backend):
                     should_strip, t.install_rpath]
                 d.targets.append(i)
 
-    def generate_pkgconfig_install(self, d):
-        pkgroot = os.path.join(self.environment.coredata.get_builtin_option('libdir'), 'pkgconfig')
-
-        for p in self.build.pkgconfig_gens:
-            pcfile = p.filebase + '.pc'
-            srcabs = os.path.join(self.environment.get_scratch_dir(),
-                                  pcfile)
-            dstrel = os.path.join(pkgroot, pcfile)
-            i = [srcabs, dstrel]
-            d.man.append(i)
-
     def generate_custom_install_script(self, d):
         d.install_scripts = self.build.install_scripts
 
@@ -511,13 +540,14 @@ class NinjaBackend(backends.Backend):
     def generate_data_install(self, d):
         data = self.build.get_data()
         for de in data:
-            subdir = de.get_install_dir()
-            for f in de.get_sources():
+            assert(isinstance(de, build.Data))
+            subdir = de.install_dir
+            for f in de.sources:
                 if de.in_sourcetree:
                     srcprefix = self.environment.get_source_dir()
                 else:
                     srcprefix = self.environment.get_build_dir()
-                srcabs = os.path.join(srcprefix, de.get_source_subdir(), f)
+                srcabs = os.path.join(srcprefix, de.source_subdir, f)
                 dstabs = os.path.join(subdir, f)
                 i = [srcabs, dstabs]
                 d.data.append(i)
@@ -528,6 +558,25 @@ class NinjaBackend(backends.Backend):
             dst_dir = os.path.join(self.environment.get_prefix(), sd.install_dir)
             d.install_subdirs.append([src_dir, dst_dir])
 
+    def write_test_suite_targets(self, cmd, outfile):
+        suites = {}
+        for t in self.build.get_tests():
+            for s in t.suite:
+                suites[s] = True
+        suites = list(suites.keys())
+        suites.sort()
+        for s in suites:
+            if s == '':
+                visible_name = 'for top level tests'
+            else:
+                visible_name = s
+            elem = NinjaBuildElement('test-' + s, 'CUSTOM_COMMAND', ['all', 'PHONY'])
+            elem.add_item('COMMAND', cmd + ['--suite=' + s])
+            elem.add_item('DESC', 'Running test suite %s.' % visible_name)
+            elem.add_item('pool', 'console')
+            elem.write(outfile)
+            self.check_outputs(elem)
+
     def generate_tests(self, outfile):
         self.serialise_tests()
         valgrind = environment.find_valgrind()
@@ -537,10 +586,11 @@ class NinjaBackend(backends.Backend):
         cmd = [sys.executable, test_script, test_data]
         elem = NinjaBuildElement('test', 'CUSTOM_COMMAND', ['all', 'PHONY'])
         elem.add_item('COMMAND', cmd)
-        elem.add_item('DESC', 'Running test suite.')
+        elem.add_item('DESC', 'Running all tests.')
         elem.add_item('pool', 'console')
         elem.write(outfile)
         self.check_outputs(elem)
+        self.write_test_suite_targets(cmd, outfile)
 
         if valgrind:
             velem = NinjaBuildElement('test-valgrind', 'CUSTOM_COMMAND', ['all', 'PHONY'])
@@ -550,6 +600,17 @@ class NinjaBackend(backends.Backend):
             velem.write(outfile)
             self.check_outputs(velem)
 
+        # And then benchmarks.
+        benchmark_script = os.path.join(script_root, 'meson_benchmark.py')
+        benchmark_data = os.path.join(self.environment.get_scratch_dir(), 'meson_benchmark_setup.dat')
+        cmd = [sys.executable, benchmark_script, benchmark_data]
+        elem = NinjaBuildElement('benchmark', 'CUSTOM_COMMAND', ['all', 'PHONY'])
+        elem.add_item('COMMAND', cmd)
+        elem.add_item('DESC', 'Running benchmark suite.')
+        elem.add_item('pool', 'console')
+        elem.write(outfile)
+        self.check_outputs(elem)
+
     def generate_rules(self, outfile):
         outfile.write('# Rules for compiling.\n\n')
         self.generate_compile_rules(outfile)
@@ -619,7 +680,7 @@ class NinjaBackend(backends.Backend):
         if e != '':
             commands.append(main_class)
         commands.append(self.get_target_filename(target))
-        commands += ['-C', self.get_target_private_dir_abs(target)]
+        commands += ['-C', self.get_target_private_dir(target)]
         commands += class_list
         elem = NinjaBuildElement(outname_rel, jar_rule, [])
         elem.add_dep([os.path.join(self.get_target_private_dir(target), i) for i in class_list])
@@ -690,7 +751,7 @@ class NinjaBackend(backends.Backend):
         args += compiler.get_output_args(self.get_target_private_dir(target))
         rel_src = src.rel_to_builddir(self.build_to_src)
         plain_class_path = src.fname[:-4] + 'class'
-        rel_obj = os.path.join(self.get_target_private_dir_abs(target), plain_class_path)
+        rel_obj = os.path.join(self.get_target_private_dir(target), plain_class_path)
         element = NinjaBuildElement(rel_obj, compiler.get_language() + '_COMPILER', rel_src)
         element.add_item('ARGS', args)
         element.write(outfile)
@@ -712,7 +773,7 @@ class NinjaBackend(backends.Backend):
             if not s.endswith('.vala'):
                 continue
             vapibase = os.path.basename(s.fname)[:-4] + 'vapi'
-            rel_vapi = os.path.join(self.get_target_private_dir_abs(target), vapibase)
+            rel_vapi = os.path.join(self.get_target_private_dir(target), vapibase)
             args = ['--fast-vapi=' + rel_vapi]
             rel_s = s.rel_to_builddir(self.build_to_src)
             element = NinjaBuildElement(rel_vapi, valac.get_language() + '_COMPILER', rel_s)
@@ -743,7 +804,7 @@ class NinjaBackend(backends.Backend):
         for s in src:
             if not s.endswith('.vala'):
                 continue
-            args = ['-d', self.get_target_private_dir_abs(target)]
+            args = ['-d', self.get_target_private_dir(target)]
             sc = os.path.basename(s.fname)[:-4] + 'c'
             args += ['-C']
             vapi_order_deps = []
@@ -753,7 +814,7 @@ class NinjaBackend(backends.Backend):
                 (vapibase, rel_vapi) = vapi_info
                 args += ['--use-fast-vapi=' + rel_vapi]
                 vapi_order_deps.append(rel_vapi)
-            relsc = os.path.join(self.get_target_private_dir_abs(target), sc)
+            relsc = os.path.join(self.get_target_private_dir(target), sc)
             rel_s = s.rel_to_builddir(self.build_to_src)
             args += ['--deps', relsc + '.d']
             if self.environment.coredata.get_builtin_option('werror'):
@@ -823,6 +884,134 @@ class NinjaBackend(backends.Backend):
         element.write(outfile)
         self.check_outputs(element)
 
+    def swift_module_file_name(self, target):
+        return os.path.join(self.get_target_private_dir(target),
+                            self.target_swift_modulename(target) + '.swiftmodule')
+
+    def target_swift_modulename(self, target):
+        return target.name
+
+    def is_swift_target(self, target):
+        for s in target.sources:
+            if s.endswith('swift'):
+                return True
+        return False
+
+    def determine_swift_dep_modules(self, target):
+        result = []
+        for l in target.link_targets:
+            if self.is_swift_target(l):
+                result.append(self.swift_module_file_name(l))
+        return result
+
+    def determine_swift_dep_dirs(self, target):
+        result = []
+        for l in target.link_targets:
+            result.append(self.get_target_private_dir_abs(l))
+        return result
+
+    def get_swift_link_deps(self, target):
+        result = []
+        for l in target.link_targets:
+            result.append(self.get_target_filename(l))
+        return result
+
+    def split_swift_generated_sources(self, target):
+        all_srcs = []
+        for genlist in target.get_generated_sources():
+            if isinstance(genlist, build.CustomTarget):
+                for ifile in genlist.get_filename():
+                    rel = os.path.join(self.get_target_dir(genlist), ifile)
+                    all_srcs.append(rel)
+            else:
+                for ifile in genlist.get_outfilelist():
+                    rel = os.path.join(self.get_target_private_dir(target), ifile)
+                    all_srcs.append(rel)
+        srcs = []
+        others = []
+        for i in all_srcs:
+            if i.endswith('.swift'):
+                srcs.append(i)
+            else:
+                others.append(i)
+        return (srcs, others)
+
+    def generate_swift_target(self, target, outfile):
+        module_name = self.target_swift_modulename(target)
+        swiftc = self.environment.coredata.compilers['swift']
+        abssrc = []
+        abs_headers = []
+        header_imports = []
+        for i in target.get_sources():
+            if swiftc.can_compile(i):
+                relsrc = i.rel_to_builddir(self.build_to_src)
+                abss = os.path.normpath(os.path.join(self.environment.get_build_dir(), relsrc))
+                abssrc.append(abss)
+            elif self.environment.is_header(i):
+                relh = i.rel_to_builddir(self.build_to_src)
+                absh = os.path.normpath(os.path.join(self.environment.get_build_dir(), relh))
+                abs_headers.append(absh)
+                header_imports += swiftc.get_header_import_args(absh)
+            else:
+                raise InvalidArguments('Swift target %s contains a non-swift source file.' % target.get_basename())
+        os.makedirs(self.get_target_private_dir_abs(target), exist_ok=True)
+        compile_args = swiftc.get_compile_only_args()
+        compile_args += swiftc.get_module_args(module_name)
+        link_args = swiftc.get_output_args(os.path.join(self.environment.get_build_dir(), self.get_target_filename(target)))
+        rundir = self.get_target_private_dir(target)
+        out_module_name = self.swift_module_file_name(target)
+        in_module_files = self.determine_swift_dep_modules(target)
+        abs_module_dirs = self.determine_swift_dep_dirs(target)
+        module_includes = []
+        for x in abs_module_dirs:
+            module_includes += swiftc.get_include_args(x)
+        link_deps = self.get_swift_link_deps(target)
+        abs_link_deps = [os.path.join(self.environment.get_build_dir(), x) for x in link_deps]
+        (rel_generated, _) = self.split_swift_generated_sources(target)
+        abs_generated = [os.path.join(self.environment.get_build_dir(), x) for x in rel_generated]
+        # We need absolute paths because swiftc needs to be invoked in a subdir
+        # and this is the easiest way about it.
+        objects = [] # Relative to swift invocation dir
+        rel_objects = [] # Relative to build.ninja
+        for i in abssrc + abs_generated:
+            base = os.path.split(i)[1]
+            oname = os.path.splitext(base)[0] + '.o'
+            objects.append(oname)
+            rel_objects.append(os.path.join(self.get_target_private_dir(target), oname))
+
+        # Swiftc does not seem to be able to emit objects and module files in one go.
+        elem = NinjaBuildElement(rel_objects,
+                                 'swift_COMPILER',
+                                 abssrc)
+        elem.add_dep(in_module_files + rel_generated)
+        elem.add_dep(abs_headers)
+        elem.add_item('ARGS', compile_args + header_imports + abs_generated + module_includes)
+        elem.add_item('RUNDIR', rundir)
+        elem.write(outfile)
+        self.check_outputs(elem)
+        elem = NinjaBuildElement(out_module_name,
+                                 'swift_COMPILER',
+                                 abssrc)
+        elem.add_dep(in_module_files + rel_generated)
+        elem.add_item('ARGS', compile_args + abs_generated + module_includes + swiftc.get_mod_gen_args())
+        elem.add_item('RUNDIR', rundir)
+        elem.write(outfile)
+        self.check_outputs(elem)
+        if isinstance(target, build.StaticLibrary):
+            elem = self.generate_link(target, outfile, self.get_target_filename(target),
+                               rel_objects, self.build.static_linker)
+            elem.write(outfile)
+        elif isinstance(target, build.Executable):
+            elem = NinjaBuildElement(self.get_target_filename(target), 'swift_COMPILER', [])
+            elem.add_dep(rel_objects)
+            elem.add_dep(link_deps)
+            elem.add_item('ARGS', link_args + swiftc.get_std_exe_link_args() + objects + abs_link_deps)
+            elem.add_item('RUNDIR', rundir)
+            elem.write(outfile)
+            self.check_outputs(elem)
+        else:
+            raise MesonException('Swift supports only executable and static library targets.')
+
     def generate_static_link_rules(self, is_cross, outfile):
         if self.build.has_language('java'):
             if not is_cross:
@@ -960,6 +1149,19 @@ class NinjaBackend(backends.Backend):
         outfile.write(depstyle)
         outfile.write('\n')
 
+    def generate_swift_compile_rules(self, compiler, outfile):
+        rule = 'rule %s_COMPILER\n' % compiler.get_language()
+        full_exe = [sys.executable,
+                    os.path.join(self.environment.get_script_dir(), 'dirchanger.py'),
+                    '$RUNDIR'] + compiler.get_exelist()
+        invoc = ' '.join([ninja_quote(i) for i in full_exe])
+        command = ' command = %s $ARGS $in\n' % invoc
+        description = ' description = Compiling Swift source $in.\n'
+        outfile.write(rule)
+        outfile.write(command)
+        outfile.write(description)
+        outfile.write('\n')
+
     def generate_fortran_dep_hack(self, outfile):
         if mesonlib.is_windows():
             cmd = 'cmd /C ""'
@@ -993,6 +1195,10 @@ rule FORTRAN_DEP_HACK
             if not is_cross:
                 self.generate_rust_compile_rules(compiler, outfile)
             return
+        if langname == 'swift':
+            if not is_cross:
+                self.generate_swift_compile_rules(compiler, outfile)
+            return
         if langname == 'fortran':
             self.generate_fortran_dep_hack(outfile)
         if is_cross:
@@ -1125,23 +1331,29 @@ rule FORTRAN_DEP_HACK
             extra_dependencies = [os.path.join(self.build_to_src, i) for i in genlist.extra_depends]
             for i in range(len(infilelist)):
                 if len(generator.outputs) == 1:
-                    sole_output = os.path.join(self.get_target_private_dir_abs(target), outfilelist[i])
+                    sole_output = os.path.join(self.get_target_private_dir(target), outfilelist[i])
                 else:
                     sole_output = ''
                 curfile = infilelist[i]
                 infilename = os.path.join(self.build_to_src, curfile)
                 outfiles = genlist.get_outputs_for(curfile)
-                outfiles = [os.path.join(self.get_target_private_dir_abs(target), of) for of in outfiles]
+                outfiles = [os.path.join(self.get_target_private_dir(target), of) for of in outfiles]
                 args = [x.replace("@INPUT@", infilename).replace('@OUTPUT@', sole_output)\
                         for x in base_args]
-                args = self.replace_outputs(args, self.get_target_private_dir_abs(target), outfilelist)
+                args = self.replace_outputs(args, self.get_target_private_dir(target), outfilelist)
                 # We have consumed output files, so drop them from the list of remaining outputs.
                 if sole_output == '':
                     outfilelist = outfilelist[len(generator.outputs):]
                 relout = self.get_target_private_dir(target)
                 args = [x.replace("@SOURCE_DIR@", self.build_to_src).replace("@BUILD_DIR@", relout)
                         for x in args]
-                cmdlist = exe_arr + args
+                final_args = []
+                for a in args:
+                    if a == '@EXTRA_ARGS@':
+                        final_args += genlist.get_extra_args()
+                    else:
+                        final_args.append(a)
+                cmdlist = exe_arr + final_args
                 elem = NinjaBuildElement(outfiles, 'CUSTOM_COMMAND', infilename)
                 if len(extra_dependencies) > 0:
                     elem.add_dep(extra_dependencies)
@@ -1183,7 +1395,7 @@ rule FORTRAN_DEP_HACK
     def get_fortran_deps(self, compiler, src, target):
         mod_files = []
         usere = re.compile(r"\s*use\s+(\w+)", re.IGNORECASE)
-        dirname = self.get_target_private_dir_abs(target)
+        dirname = self.get_target_private_dir(target)
         tdeps= self.fortran_deps[target.get_basename()]
         for line in open(src):
             usematch = usere.match(line)
@@ -1233,7 +1445,7 @@ rule FORTRAN_DEP_HACK
             if self.has_dir_part(src):
                 rel_src = src
             else:
-                rel_src = os.path.join(self.get_target_private_dir_abs(target), src)
+                rel_src = os.path.join(self.get_target_private_dir(target), src)
                 abs_src = os.path.join(self.environment.get_source_dir(), rel_src)
         else:
             if isinstance(src, File):
@@ -1252,7 +1464,7 @@ rule FORTRAN_DEP_HACK
         obj_basename = src_filename.replace('/', '_').replace('\\', '_')
         rel_obj = os.path.join(self.get_target_private_dir(target), obj_basename)
         rel_obj += '.' + self.environment.get_object_suffix()
-        dep_file = rel_obj + '.' + compiler.get_depfile_suffix()
+        dep_file = compiler.depfile_for_object(rel_obj)
         if self.environment.coredata.get_builtin_option('use_pch'):
             pchlist = target.get_pch(compiler.language)
         else:
@@ -1261,7 +1473,7 @@ rule FORTRAN_DEP_HACK
             pch_dep = []
         else:
             arr = []
-            i = os.path.join(self.get_target_private_dir_abs(target), compiler.get_pch_name(pchlist[0]))
+            i = os.path.join(self.get_target_private_dir(target), compiler.get_pch_name(pchlist[0]))
             arr.append(i)
             pch_dep = arr
         for i in target.get_include_dirs():
@@ -1301,14 +1513,14 @@ rule FORTRAN_DEP_HACK
                     depelem = NinjaBuildElement(modfile, 'FORTRAN_DEP_HACK', rel_obj)
                     depelem.write(outfile)
                     self.check_outputs(depelem)
-            commands += compiler.get_module_outdir_args(self.get_target_private_dir_abs(target))
+            commands += compiler.get_module_outdir_args(self.get_target_private_dir(target))
 
         element = NinjaBuildElement(rel_obj, compiler_name, rel_src)
         for d in header_deps:
             if isinstance(d, RawFilename):
                 d = d.fname
             elif not self.has_dir_part(d):
-                d = os.path.join(self.get_target_private_dir_abs(target), d)
+                d = os.path.join(self.get_target_private_dir(target), d)
             element.add_dep(d)
         for d in extra_deps:
             element.add_dep(d)
@@ -1316,7 +1528,7 @@ rule FORTRAN_DEP_HACK
             if isinstance(d, RawFilename):
                 d = d.fname
             elif not self.has_dir_part(d):
-                d = os.path.join(self.get_target_private_dir_abs(target), d)
+                d = os.path.join(self.get_target_private_dir(target), d)
             element.add_orderdep(d)
         element.add_orderdep(pch_dep)
         element.add_orderdep(extra_orderdeps)
@@ -1348,7 +1560,7 @@ rule FORTRAN_DEP_HACK
         header = pch[0]
         source = pch[1]
         pchname = compiler.get_pch_name(header)
-        dst = os.path.join(self.get_target_private_dir_abs(target), pchname)
+        dst = os.path.join(self.get_target_private_dir(target), pchname)
 
         commands = []
         commands += self.generate_basic_compiler_args(target, compiler)
@@ -1361,7 +1573,7 @@ rule FORTRAN_DEP_HACK
     def generate_gcc_pch_command(self, target, compiler, pch):
         commands = []
         commands += self.generate_basic_compiler_args(target, compiler)
-        dst = os.path.join(self.get_target_private_dir_abs(target),
+        dst = os.path.join(self.get_target_private_dir(target),
                            os.path.split(pch)[-1] + '.' + compiler.get_pch_suffix())
         dep = dst + '.' + compiler.get_depfile_suffix()
         return (commands, dep, dst, []) # Gcc does not create an object file during pch generation.
@@ -1399,7 +1611,7 @@ rule FORTRAN_DEP_HACK
 
     def generate_shsym(self, outfile, target):
         target_name = self.get_target_filename(target)
-        targetdir = self.get_target_private_dir_abs(target)
+        targetdir = self.get_target_private_dir(target)
         symname = os.path.join(targetdir, target_name + '.symbols')
         elem = NinjaBuildElement(symname, 'SHSYM', target_name)
         if self.environment.is_cross_build() and self.environment.cross_info.need_cross_compiler():
@@ -1464,16 +1676,29 @@ rule FORTRAN_DEP_HACK
                                             self.determine_rpath_dirs(target), target.install_rpath)
         if self.environment.coredata.get_builtin_option('coverage'):
             commands += linker.get_coverage_link_args()
+        custom_target_libraries = self.get_custom_target_provided_libraries(target)
         commands += extra_args
+        commands += custom_target_libraries
+        commands = linker.unixtype_flags_to_native(commands)
         dep_targets = [self.get_dependency_filename(t) for t in dependencies]
         dep_targets += [os.path.join(self.environment.source_dir,
                                      target.subdir, t) for t in target.link_depends]
         elem = NinjaBuildElement(outname, linker_rule, obj_list)
-        elem.add_dep(dep_targets)
+        elem.add_dep(dep_targets + custom_target_libraries)
         elem.add_item('LINK_ARGS', commands)
         self.check_outputs(elem)
         return elem
 
+    def get_custom_target_provided_libraries(self, target):
+        libs = []
+        for t in target.get_generated_sources():
+            if not isinstance(t, build.CustomTarget):
+                continue
+            for f in t.output:
+                if self.environment.is_library(f):
+                    libs.append(os.path.join(self.get_target_dir(t), f))
+        return libs
+
     def determine_rpath_dirs(self, target):
         link_deps = target.get_all_link_deps()
         result = []
@@ -1485,7 +1710,7 @@ rule FORTRAN_DEP_HACK
 
     def get_dependency_filename(self, t):
         if isinstance(t, build.SharedLibrary):
-            return os.path.join(self.get_target_private_dir_abs(t), self.get_target_filename(t) + '.symbols')
+            return os.path.join(self.get_target_private_dir(t), self.get_target_filename(t) + '.symbols')
         return self.get_target_filename(t)
 
     def generate_shlib_aliases(self, target, outdir):
@@ -1539,7 +1764,7 @@ rule FORTRAN_DEP_HACK
                 plainname = os.path.split(src)[1]
                 basename = plainname.split('.')[0]
                 outname = rule.name_templ.replace('@BASENAME@', basename).replace('@PLAINNAME@', plainname)
-                outfilename = os.path.join(self.get_target_private_dir_abs(target), outname)
+                outfilename = os.path.join(self.get_target_private_dir(target), outname)
                 infilename = os.path.join(self.build_to_src, target.get_source_subdir(), src)
                 elem = NinjaBuildElement(outfilename, rule.name, infilename)
                 elem.write(outfile)
