@@ -21,7 +21,7 @@ import sys
 import environment
 import mesonlib
 import mlog
-import meson, meson_test
+import meson, meson_test, meson_benchmark
 import argparse
 import xml.etree.ElementTree as ET
 import time
@@ -87,7 +87,7 @@ def setup_commands(backend):
             compile_commands = [ninja_command, '-v']
         else:
             compile_commands = [ninja_command]
-        test_commands = [ninja_command, 'test']
+        test_commands = [ninja_command, 'test', 'benchmark']
         install_commands = [ninja_command, 'install']
 
 def platform_fix_filename(fname):
@@ -153,9 +153,11 @@ def run_configure_inprocess(commandlist):
     sys.stdout = mystdout = StringIO()
     old_stderr = sys.stderr
     sys.stderr = mystderr = StringIO()
-    returncode = meson.run(commandlist)
-    sys.stdout = old_stdout
-    sys.stderr = old_stderr
+    try:
+        returncode = meson.run(commandlist)
+    finally:
+        sys.stdout = old_stdout
+        sys.stderr = old_stderr
     return (returncode, mystdout.getvalue(), mystderr.getvalue())
 
 def run_test_inprocess(testdir):
@@ -165,11 +167,14 @@ def run_test_inprocess(testdir):
     sys.stderr = mystderr = StringIO()
     old_cwd = os.getcwd()
     os.chdir(testdir)
-    returncode = meson_test.run(['meson-private/meson_test_setup.dat'])
-    sys.stdout = old_stdout
-    sys.stderr = old_stderr
-    os.chdir(old_cwd)
-    return (returncode, mystdout.getvalue(), mystderr.getvalue())
+    try:
+        returncode_test = meson_test.run(['meson-private/meson_test_setup.dat'])
+        returncode_benchmark = meson_benchmark.run(['meson-private/meson_benchmark_setup.dat'])
+    finally:
+        sys.stdout = old_stdout
+        sys.stderr = old_stderr
+        os.chdir(old_cwd)
+    return (max(returncode_test, returncode_benchmark), mystdout.getvalue(), mystderr.getvalue())
 
 
 def run_test(testdir, extra_args, should_succeed):
@@ -253,6 +258,7 @@ def detect_tests_to_run():
     all_tests.append(('rust', gather_tests('test cases/rust'), False if shutil.which('rustc') else True))
     all_tests.append(('objective c', gather_tests('test cases/objc'), False if not mesonlib.is_windows() else True))
     all_tests.append(('fortran', gather_tests('test cases/fortran'), False if shutil.which('gfortran') else True))
+    all_tests.append(('swift', gather_tests('test cases/swift'), False if shutil.which('swiftc') else True))
     return all_tests
 
 def run_tests(extra_args):
